from __future__ import annotations

import typing

import borsh_construct as borsh
from solders.instruction import AccountMeta, Instruction
from solders.pubkey import Pubkey

from .. import types
from ..program_id import PROGRAM_ID


class OverrideExpiryArgs(typing.TypedDict):
    args: types.override_expiry_args.OverrideExpiryArgs


layout = borsh.CStruct("args" / types.override_expiry_args.OverrideExpiryArgs.layout)


class OverrideExpiryAccounts(typing.TypedDict):
    state: Pubkey
    admin: Pubkey
    zeta_group: Pubkey


def override_expiry(
    args: OverrideExpiryArgs,
    accounts: OverrideExpiryAccounts,
    program_id: Pubkey = PROGRAM_ID,
    remaining_accounts: typing.Optional[typing.List[AccountMeta]] = None,
) -> Instruction:
    keys: list[AccountMeta] = [
        AccountMeta(pubkey=accounts["state"], is_signer=False, is_writable=False),
        AccountMeta(pubkey=accounts["admin"], is_signer=True, is_writable=False),
        AccountMeta(pubkey=accounts["zeta_group"], is_signer=False, is_writable=True),
    ]
    if remaining_accounts is not None:
        keys += remaining_accounts
    identifier = b"\x81\xc5urlw\xcf\x88"
    encoded_args = layout.build(
        {
            "args": args["args"].to_encodable(),
        }
    )
    data = identifier + encoded_args
    return Instruction(program_id, data, keys)



from __future__ import annotations

import typing

import borsh_construct as borsh
from solders.instruction import AccountMeta, Instruction
from solders.pubkey import Pubkey
from solders.system_program import ID as SYS_PROGRAM_ID

from .. import types
from ..program_id import PROGRAM_ID


class InitializeMarketNodeArgs(typing.TypedDict):
    args: types.initialize_market_node_args.InitializeMarketNodeArgs


layout = borsh.CStruct("args" / types.initialize_market_node_args.InitializeMarketNodeArgs.layout)


class InitializeMarketNodeAccounts(typing.TypedDict):
    zeta_group: Pubkey
    market_node: Pubkey
    greeks: Pubkey
    payer: Pubkey


def initialize_market_node(
    args: InitializeMarketNodeArgs,
    accounts: InitializeMarketNodeAccounts,
    program_id: Pubkey = PROGRAM_ID,
    remaining_accounts: typing.Optional[typing.List[AccountMeta]] = None,
) -> Instruction:
    keys: list[AccountMeta] = [
        AccountMeta(pubkey=accounts["zeta_group"], is_signer=False, is_writable=False),
        AccountMeta(pubkey=accounts["market_node"], is_signer=False, is_writable=True),
        AccountMeta(pubkey=accounts["greeks"], is_signer=False, is_writable=True),
        AccountMeta(pubkey=accounts["payer"], is_signer=True, is_writable=True),
        AccountMeta(pubkey=SYS_PROGRAM_ID, is_signer=False, is_writable=False),
    ]
    if remaining_accounts is not None:
        keys += remaining_accounts
    identifier = b"2v\x15\x15\xb3\xf8\x17\x80"
    encoded_args = layout.build(
        {
            "args": args["args"].to_encodable(),
        }
    )
    data = identifier + encoded_args
    return Instruction(program_id, data, keys)



from __future__ import annotations

import typing

import borsh_construct as borsh
from anchorpy.borsh_extension import BorshPubkey
from solders.instruction import AccountMeta, Instruction
from solders.pubkey import Pubkey
from solders.system_program import ID as SYS_PROGRAM_ID

from ..program_id import PROGRAM_ID


class InitializeCrossMarginAccountManagerV2Args(typing.TypedDict):
    referrer: typing.Optional[Pubkey]


layout = borsh.CStruct("referrer" / borsh.Option(BorshPubkey))


class InitializeCrossMarginAccountManagerV2Accounts(typing.TypedDict):
    cross_margin_account_manager: Pubkey
    authority: Pubkey
    payer: Pubkey
    zeta_program: Pubkey


def initialize_cross_margin_account_manager_v2(
    args: InitializeCrossMarginAccountManagerV2Args,
    accounts: InitializeCrossMarginAccountManagerV2Accounts,
    program_id: Pubkey = PROGRAM_ID,
    remaining_accounts: typing.Optional[typing.List[AccountMeta]] = None,
) -> Instruction:
    keys: list[AccountMeta] = [
        AccountMeta(
            pubkey=accounts["cross_margin_account_manager"],
            is_signer=False,
            is_writable=True,
        ),
        AccountMeta(pubkey=accounts["authority"], is_signer=False, is_writable=False),
        AccountMeta(pubkey=accounts["payer"], is_signer=True, is_writable=True),
        AccountMeta(pubkey=accounts["zeta_program"], is_signer=False, is_writable=False),
        AccountMeta(pubkey=SYS_PROGRAM_ID, is_signer=False, is_writable=False),
    ]
    if remaining_accounts is not None:
        keys += remaining_accounts
    identifier = b"\xa6\xcb\xb0\xd2\xb04\x8ci"
    encoded_args = layout.build(
        {
            "referrer": args["referrer"],
        }
    )
    data = identifier + encoded_args
    return Instruction(program_id, data, keys)



from __future__ import annotations

import typing

import borsh_construct as borsh
from solders.instruction import AccountMeta, Instruction
from solders.pubkey import Pubkey

from ..program_id import PROGRAM_ID


class CloseOpenOrdersV2Args(typing.TypedDict):
    map_nonce: int


layout = borsh.CStruct("map_nonce" / borsh.U8)


class CloseOpenOrdersV2Accounts(typing.TypedDict):
    state: Pubkey
    dex_program: Pubkey
    open_orders: Pubkey
    margin_account: Pubkey
    authority: Pubkey
    market: Pubkey
    serum_authority: Pubkey
    open_orders_map: Pubkey


def close_open_orders_v2(
    args: CloseOpenOrdersV2Args,
    accounts: CloseOpenOrdersV2Accounts,
    program_id: Pubkey = PROGRAM_ID,
    remaining_accounts: typing.Optional[typing.List[AccountMeta]] = None,
) -> Instruction:
    keys: list[AccountMeta] = [
        AccountMeta(pubkey=accounts["state"], is_signer=False, is_writable=False),
        AccountMeta(pubkey=accounts["dex_program"], is_signer=False, is_writable=False),
        AccountMeta(pubkey=accounts["open_orders"], is_signer=False, is_writable=True),
        AccountMeta(pubkey=accounts["margin_account"], is_signer=False, is_writable=True),
        AccountMeta(pubkey=accounts["authority"], is_signer=True, is_writable=True),
        AccountMeta(pubkey=accounts["market"], is_signer=False, is_writable=False),
        AccountMeta(pubkey=accounts["serum_authority"], is_signer=False, is_writable=False),
        AccountMeta(pubkey=accounts["open_orders_map"], is_signer=False, is_writable=True),
    ]
    if remaining_accounts is not None:
        keys += remaining_accounts
    identifier = b"N\x98\xc4\xa3D\xb3yH"
    encoded_args = layout.build(
        {
            "map_nonce": args["map_nonce"],
        }
    )
    data = identifier + encoded_args
    return Instruction(program_id, data, keys)



from __future__ import annotations

import typing

import borsh_construct as borsh
from solders.instruction import AccountMeta, Instruction
from solders.pubkey import Pubkey

from ..program_id import PROGRAM_ID


class UpdateMakerTradeFeePercentageArgs(typing.TypedDict):
    new_native_maker_trade_fee_percentage: int


layout = borsh.CStruct("new_native_maker_trade_fee_percentage" / borsh.U64)


class UpdateMakerTradeFeePercentageAccounts(typing.TypedDict):
    state: Pubkey
    admin: Pubkey


def update_maker_trade_fee_percentage(
    args: UpdateMakerTradeFeePercentageArgs,
    accounts: UpdateMakerTradeFeePercentageAccounts,
    program_id: Pubkey = PROGRAM_ID,
    remaining_accounts: typing.Optional[typing.List[AccountMeta]] = None,
) -> Instruction:
    keys: list[AccountMeta] = [
        AccountMeta(pubkey=accounts["state"], is_signer=False, is_writable=True),
        AccountMeta(pubkey=accounts["admin"], is_signer=True, is_writable=False),
    ]
    if remaining_accounts is not None:
        keys += remaining_accounts
    identifier = b"8>y\xd2\xa6\xce&\xda"
    encoded_args = layout.build(
        {
            "new_native_maker_trade_fee_percentage": args["new_native_maker_trade_fee_percentage"],
        }
    )
    data = identifier + encoded_args
    return Instruction(program_id, data, keys)
